/*******************************************************************************
 * Copyright (c) 2006 Oracle Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Cameron Bateman/Oracle - initial API and implementation
 *    
 ********************************************************************************/

package org.eclipse.jst.jsf.designtime.internal.provisional.el;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import org.eclipse.jst.jsf.context.symbol.internal.provisional.IObjectSymbol;
import org.eclipse.jst.jsf.context.symbol.internal.provisional.ISymbol;

/**
 * A design time proxy for the runtime PropertyResolver.  This is used to
 * resolve all but the first element of a var.prop.prop2 type of sub-expression in
 * a JSF EL expression. @see DefaultDTVariableResolver for how to resolve 'var' at
 * designtime
 *
 * Clients may implement
 * 
 * @author cbateman
 */
public class DefaultDTPropertyResolver extends AbstractDTPropertyResolver
{
    /**
     * Returns a symbol encapsulating the property on base with the name
     * properyId
     * 
     * @param base
     * @param propertyId
     * @return the symbol for the named propertyId or null if not found
     */
    public ISymbol getProperty(ISymbol base, Object propertyId)
    {
        for (final Iterator it = getIterator(base); it.hasNext();)
        {
            ISymbol element = (ISymbol) it.next();
            if (element.getName().equals(propertyId))
            {
                return element;
            }
        }

        return null;
    }
    
    /**
     * @param base
     * @return all properties of base
     */
    public ISymbol[] getAllProperties(ISymbol base)
    {
        return (ISymbol[]) getAllPropertiesList(base).
                    toArray(ISymbol.EMPTY_SYMBOL_ARRAY);
    }
    
    private List getAllPropertiesList(ISymbol symbol)
    {
        if (symbol instanceof IObjectSymbol)
        {
            return ((IObjectSymbol)symbol).getTypeDescriptor().
                getProperties();
        }

        // otherwise, the symbol has no properties
        return Collections.EMPTY_LIST;
    }
    
    private Iterator getIterator(ISymbol symbol)
    {
        return getAllPropertiesList(symbol).iterator();
    }
}
