/*******************************************************************************
 * Copyright (c) 2004, 2005 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/
package org.eclipse.jst.jsf.facesconfig.common.dialogfield;

import java.util.List;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IClassFile;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragment;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaConventions;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.ui.wizards.NewClassWizardPage;

/**
 * @author zpzhang
 */
public class JavaClassWizardPage extends NewClassWizardPage {
	private String className;

	private IProject project;

	private InitialClassProperties initialValues;

	private IJavaProject javaProject;

	private IStatus fClassNameStatus, fPackageNameStatus;

	class InitialClassProperties {
		// populate new wizard page
		IType superClassType;

		String superClassName;

		List interfacesName;

		String theClassName;

		String classArgs;

		String packageName;

		IPackageFragmentRoot packageFragmentRoot;

		IPackageFragment packageFragment;

		public InitialClassProperties() {
			this.superClassType = null;
			this.superClassName = ""; //$NON-NLS-1$
			this.interfacesName = null;
			this.theClassName = null;
			this.classArgs = null;
			this.packageName = null;
			this.packageFragment = null;
			this.packageFragmentRoot = null;
		}
	}

	public JavaClassWizardPage(IProject project, String theClassName,
			String superClassName, List interfacesName) {
		super();
		this.className = theClassName;
		this.project = project;
		try {
			if (project != null && project.hasNature(JavaCore.NATURE_ID)) {
				this.javaProject = JavaCore.create(project);
			} else {
				this.javaProject = null;
			}
		} catch (CoreException e) {
			e.printStackTrace();// PDEPlugin.logException(e);
		}
		initialValues = new InitialClassProperties();
		initialValues.theClassName = theClassName;
		initialValues.superClassName = superClassName;
		initialValues.interfacesName = interfacesName;
	}

	public JavaClassWizardPage(IProject project, String className) {
		this(project, className, null, null);
	}

	public void init() {
		initializeExpectedValues();
		initializeWizardPage();
	}

	private void initializeExpectedValues() {
		// source folder name, package name, class name
		int loc = className.indexOf(":"); //$NON-NLS-1$
		if (loc != -1) {
			if (loc < className.length()) {
				initialValues.classArgs = className.substring(loc + 1,
						className.length());
				className = className.substring(0, loc);
			}
			if (loc > 0) {
				initialValues.theClassName = className.substring(0, loc);
			} else if (loc == 0) {
				initialValues.theClassName = ""; //$NON-NLS-1$
			}
		}
		fClassNameStatus = JavaConventions
				.validateJavaTypeName(initialValues.theClassName);

		loc = className.lastIndexOf('.');
		if (loc != -1) {
			initialValues.packageName = className.substring(0, loc);
			initialValues.theClassName = className.substring(loc + 1);
			fPackageNameStatus = JavaConventions
					.validatePackageName(initialValues.packageName);
			fClassNameStatus = JavaConventions
					.validateJavaTypeName(initialValues.theClassName);
		}
		if (javaProject == null) {
			return;
		}
		try {
			if (initialValues.packageFragmentRoot == null) {
				IPackageFragmentRoot srcEntryDft = null;
				IPackageFragmentRoot[] roots = javaProject
						.getPackageFragmentRoots();
				for (int i = 0; i < roots.length; i++) {
					if (roots[i].getKind() == IPackageFragmentRoot.K_SOURCE) {
						srcEntryDft = roots[i];
						break;
					}
				}
				if (srcEntryDft != null) {
					initialValues.packageFragmentRoot = srcEntryDft;
				} else {
					initialValues.packageFragmentRoot = javaProject
							.getPackageFragmentRoot(javaProject.getResource());
				}
				if (initialValues.packageFragment == null
						&& initialValues.packageFragmentRoot != null
						&& initialValues.packageName != null
						&& initialValues.packageName.length() > 0) {
					IFolder packageFolder = project
							.getFolder(initialValues.packageName);
					initialValues.packageFragment = initialValues.packageFragmentRoot
							.getPackageFragment(packageFolder
									.getProjectRelativePath().toOSString());
				}
			}
			// superclass and interface
			if (initialValues.superClassName == null) {
				initialValues.superClassName = "java.lang.Object"; //$NON-NLS-1$
			}
			initialValues.superClassType = findTypeForName(initialValues.superClassName);
		} catch (JavaModelException e) {
			e.printStackTrace();// PDEPlugin.logException(e);
		}
	}

	protected void initializeWizardPage() {
		setPackageFragmentRoot(initialValues.packageFragmentRoot, true);
		setPackageFragment(initialValues.packageFragment, true);
		setEnclosingType(null, true);
		setEnclosingTypeSelection(false, true);
		setTypeName(initialValues.theClassName, true);
		setSuperClass(initialValues.superClassName, true);
		if (initialValues.interfacesName != null) {
			setSuperInterfaces(initialValues.interfacesName, true);
		}
		boolean hasSuperClass = initialValues.superClassName != null
				&& initialValues.superClassName.length() > 0;
		boolean hasInterface = initialValues.interfacesName != null
				&& initialValues.interfacesName.size() > 0;
		setMethodStubSelection(false, hasSuperClass, hasInterface
				|| hasSuperClass, true);
	}

	private IType findTypeForName(String typeName) throws JavaModelException {
		if (typeName == null || typeName.length() == 0) {
			return null;
		}
		IType type = null;
		String fileName = typeName.replace('.', '/') + ".java"; //$NON-NLS-1$
		IJavaElement element = javaProject.findElement(new Path(fileName));
		if (element == null) {
			return null;
		}
		if (element instanceof IClassFile) {
			type = ((IClassFile) element).getType();
		} else if (element instanceof ICompilationUnit) {
			IType[] types = ((ICompilationUnit) element).getTypes();
			type = types[0];
		}
		return type;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jdt.ui.wizards.NewClassWizardPage#setVisible(boolean)
	 */
	public void setVisible(boolean visible) {
		super.setVisible(visible);
		// policy: wizards are not allowed to come up with an error message;
		// in this wizard, some fields may need initial validation and thus,
		// potentially start with an error message.
		if (fClassNameStatus != null && !fClassNameStatus.isOK()) {
			updateStatus(fClassNameStatus);
		}
		if (fPackageNameStatus != null && !fPackageNameStatus.isOK()) {
			updateStatus(fPackageNameStatus);
		}
	}

	public String getClassArgs() {
		if (initialValues.classArgs == null) {
			return ""; //$NON-NLS-1$
		}
		return initialValues.classArgs;
	}

}
