/*******************************************************************************
 * Copyright (c) 2004, 2005 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/

package org.eclipse.jst.jsf.facesconfig.common.guiutils;

import org.eclipse.jface.window.ApplicationWindow;
import org.eclipse.jst.jsf.facesconfig.common.CommonPlugin;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.LocationEvent;
import org.eclipse.swt.browser.LocationListener;
import org.eclipse.swt.browser.OpenWindowListener;
import org.eclipse.swt.browser.StatusTextEvent;
import org.eclipse.swt.browser.StatusTextListener;
import org.eclipse.swt.browser.WindowEvent;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;


/**
 * Brings up a browser into a separate window.
 * 
 * @author collinsc
 */
public class BrowserWindow extends ApplicationWindow {
	/**
	 * Command to send to the browser.
	 */
	public static final int BROWSER_CMD_BACK = 1;

	public static final int BROWSER_CMD_FORWARD = 2;

	public static final int BROWSER_CMD_STOP = 3;

	public static final int BROWSER_CMD_REFRESH = 4;

	public static final int BROWSER_CMD_RELOAD = 5;

	protected boolean bIncludeToolbar;

	protected boolean bPopups;

	protected String title;

	protected ScrolledComposite scomp;

	protected Composite browserComposite;

	protected Browser browser;

	protected String startLocation;

	protected String startContent;

	protected Point size = new Point(600, 600);

	// these items are browser context sensitive.
	protected ToolItem backItem;

	protected ToolItem forwardItem;

	/**
	 * @param parentShell
	 *            owner of this window.
	 */
	public BrowserWindow(Shell parentShell) {
		this(parentShell, false, null, false);
	}

	/**
	 * @param parentShell
	 *            owner of this window.
	 * @param bPopups
	 *            allow popup windows?
	 * @param title
	 *            Tittle on window.
	 */
	public BrowserWindow(Shell parentShell, boolean bPopups, String title) {
		this(parentShell, bPopups, title, false);
	}

	/**
	 * @param parentShell
	 *            owner of this window.
	 * @param bPopups
	 *            allow popup windows?
	 * @param title
	 *            Tittle on window.
	 * @param bModal
	 *            pretend to a be a dialog?
	 */
	public BrowserWindow(Shell parentShell, boolean bPopups, String title,
			boolean bModal) {
		super(parentShell);
		addStatusLine();
		this.bPopups = bPopups;
		this.title = title;
		includeToolbar(true);

		if (bModal && (parentShell != null)) {
			// APPLICATION_MODAL, MODELESS, PRIMARY_MODAL, SYSTEM_MODAL
			setShellStyle(getShellStyle() | SWT.APPLICATION_MODAL);
			setBlockOnOpen(true);
		}
	}

	/**
	 * Default is true, set whether to include the toolbar at the top. This must
	 * be called before the window is opened.
	 * 
	 * @param bIncludeToolbar
	 */
	public void includeToolbar(boolean bIncludeToolbar) {
		this.bIncludeToolbar = bIncludeToolbar;
	}

	public void setTitle(String title) {
		if ((title != null) && (super.getShell() != null)) {
			super.getShell().setText(title);
		}
	}

	public void setStartLocation(String startLocation) {
		this.startLocation = startLocation;
	}

	public void setStartContent(String startContent) {
		this.startContent = startContent;
	}

	public Browser getBrowser() {
		return browser;
	}

	protected Control createContents(Composite parent) {
		// Create the scroll composite as the one inside the parent.
		scomp = new ScrolledComposite(parent, SWT.V_SCROLL | SWT.H_SCROLL);

		// Create the composite that goes inside the scroller. We will use this
		// for
		// everything else from here.
		Composite innerComposite = SWTUtils.createComposite(scomp, 1, 10, -1);
		scomp.setContent(innerComposite);
		ControlListener listener = new ControlListener() {
			public void controlMoved(ControlEvent e) {
			}

			// this will handle resizing the browser when the window is resized.
			public void controlResized(ControlEvent e) {
				Point pt = scomp.getParent().getSize();
				Rectangle rect = browserComposite.getBounds();

				pt.x -= rect.x;
				pt.y -= rect.y;

				// spacing offsets in the composites.
				pt.x -= 10;
				pt.y -= 60; // need status bar at bottom...30 more than normal.

				// do the sanity check here since it prevents further checks.
				if ((pt.x <= 0) || (pt.y <= 0)) {
					return;
				}

				Rectangle innerRect = scomp.getContent().getBounds();
				Rectangle outerRect = scomp.getBounds();

				// substract if the scroll bars are visible.
				if (innerRect.width > outerRect.width) {
					pt.y -= scomp.getHorizontalBar().getSize().y;
				}
				if (innerRect.height > outerRect.height) {
					pt.x -= scomp.getVerticalBar().getSize().x;
				}

				browserComposite.setSize(pt);
				browserComposite.getParent().layout(true);
			}
		};
		scomp.addControlListener(listener);

		if (bIncludeToolbar) {
			// ////////////////////////////////////////
			// toobar
			// ////////////////////////////////////////
			ToolBar tb = new ToolBar(innerComposite, SWT.HORIZONTAL | SWT.FLAT);
			fillToolbar(tb);
		}

		// ////////////////////////////////////////
		// browser
		// ////////////////////////////////////////
		browserComposite = new Composite(innerComposite, SWT.NULL);
		browserComposite.setLayout(new FillLayout());

		browser = new Browser(browserComposite, SWT.NULL);

		if (startLocation != null) {
			browser.setUrl(startLocation);
		} else if (startContent != null) {
			browser.setText(startContent);
		} else {
			browser.setText("<b>Loading...</b>");
		}

		// send the browser status messages to eclipse.
		browser.addStatusTextListener(new StatusTextListener() {
			public void changed(StatusTextEvent event) {
				setStatus(event.text);
			}
		});

		if (bIncludeToolbar) {
			// check for change in location so we can enable the back/forward
			// buttons.
			browser.addLocationListener(new LocationListener() {
				public void changing(LocationEvent event) {
				}

				public void changed(LocationEvent event) {
					// check if there is something in the forward or back queue.
					backItem.setEnabled(browser.isBackEnabled());
					forwardItem.setEnabled(browser.isForwardEnabled());
				}
			});
		}

		// check is we need to open a new window.
		browser.addOpenWindowListener(new OpenWindowListener() {
			public void open(WindowEvent event) {
				if (bPopups) {
					BrowserWindow window = new BrowserWindow(getShell());
					window.open();
					event.browser = window.getBrowser();
				} else {
					event.browser = getBrowser();
				}
			}
		});

		setTitle(title);
		SWTUtils.computeScrollArea(scomp, innerComposite);
		return scomp;
	}

	/**
	 * Fill the toolbar for this window. This method can be overwritten to place
	 * other toolbar controls in the bar.
	 * 
	 * @param tb
	 */
	protected void fillToolbar(ToolBar tb) {
		// Back browser
		backItem = new ToolItem(tb, SWT.PUSH);
		backItem
				.setImage(CommonPlugin.getDefault().getImage("back_nav_16.gif"));
		backItem.setToolTipText(CommonPlugin
				.getResourceString("BrowserWindow.back.tooltip"));
		SelectionListener backSelectionListener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				browserCmd(BROWSER_CMD_BACK);
			}
		};
		backItem.addSelectionListener(backSelectionListener);

		// Forward Browser
		forwardItem = new ToolItem(tb, SWT.PUSH);
		forwardItem.setImage(CommonPlugin.getDefault().getImage(
				"forward_nav_16.gif"));
		forwardItem.setToolTipText(CommonPlugin
				.getResourceString("BrowserWindow.forward.tooltip"));
		SelectionListener forwardSelectionListener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				browserCmd(BROWSER_CMD_FORWARD);
			}
		};
		forwardItem.addSelectionListener(forwardSelectionListener);

		// Stop Browser
		ToolItem stopItem = new ToolItem(tb, SWT.PUSH);
		stopItem
				.setImage(CommonPlugin.getDefault().getImage("stop_nav_16.gif"));
		stopItem.setToolTipText(CommonPlugin
				.getResourceString("BrowserWindow.stop.tooltip"));
		SelectionListener stopSelectionListener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				browserCmd(BROWSER_CMD_STOP);
			}
		};
		stopItem.addSelectionListener(stopSelectionListener);

		// Refresh Browser
		ToolItem refreshItem = new ToolItem(tb, SWT.PUSH);
		refreshItem.setImage(CommonPlugin.getDefault().getImage(
				"refresh_nav_16.gif"));
		refreshItem.setToolTipText(CommonPlugin
				.getResourceString("BrowserWindow.refresh.tooltip"));
		SelectionListener refreshServiceSelectionListener = new SelectionAdapter() {
			public void widgetSelected(SelectionEvent event) {
				browserCmd(BROWSER_CMD_REFRESH);
			}
		};
		refreshItem.addSelectionListener(refreshServiceSelectionListener);

		if ((startLocation != null) || (startContent != null)) {
			// Reload Browser.
			ToolItem reloadItem = new ToolItem(tb, SWT.PUSH);
			reloadItem.setImage(CommonPlugin.getDefault().getImage(
					"reload_nav_16.gif"));
			reloadItem.setToolTipText(CommonPlugin
					.getResourceString("BrowserWindow.reload.tooltip"));
			SelectionListener reloadSelectionListener = new SelectionAdapter() {
				public void widgetSelected(SelectionEvent event) {
					browserCmd(BROWSER_CMD_RELOAD);
				}
			};
			reloadItem.addSelectionListener(reloadSelectionListener);
		}
	}

	/**
	 * send the browser widget a command as defined by the cmd constant defines.
	 * 
	 * @return true if the command was successful.
	 */
	public boolean browserCmd(int cmd) {
		switch (cmd) {
		case BROWSER_CMD_BACK:
			return browser.back();
		case BROWSER_CMD_FORWARD:
			return browser.forward();
		case BROWSER_CMD_STOP:
			browser.stop();
			return true;
		case BROWSER_CMD_REFRESH:
			browser.refresh();
			return true;
		case BROWSER_CMD_RELOAD:
			if (startLocation != null) {
				browser.setUrl(startLocation);
			} else if (startContent != null) {
				browser.setText(startContent);
			}
			return true;

		}
		return false;
	}

	public void setStartSize(Point pp) {
		size = pp;
	}

	public void setStartSize(int x, int y) {
		size = new Point(x, y);
	}

	protected Point getInitialSize() {
		return size;
	}
}