/*******************************************************************************
 * Copyright (c) 2004, 2005 Sybase, Inc. and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Sybase, Inc. - initial API and implementation
 *******************************************************************************/
package org.eclipse.jst.jsf.facesconfig.common.utils;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Display;

/**
 * @author zpzhang
 * @version
 */
public class ColorUtil {
	public static final HashMap SYSTEM_DEFAULT_COLORS = new HashMap(20);

	public static final HashMap CSS_COLORS = new HashMap(20);

	static {
		CSS_COLORS.put("black", ColorConstants.black);
		CSS_COLORS.put("blue", ColorConstants.blue);
		CSS_COLORS.put("gray", ColorConstants.gray);
		CSS_COLORS.put("green", ColorConstants.green);
		CSS_COLORS.put("orange", ColorConstants.orange);
		CSS_COLORS.put("red", ColorConstants.red);
		CSS_COLORS.put("white", ColorConstants.white);
		CSS_COLORS.put("yellow", ColorConstants.yellow);
		CSS_COLORS.put("aqua", ColorConstants.cyan);
		CSS_COLORS.put("fuchsia", new Color(null, 255, 0, 255));
		CSS_COLORS.put("lime", ColorConstants.green);
		CSS_COLORS.put("maroon", new Color(null, 128, 0, 0));
		CSS_COLORS.put("navy", new Color(null, 0, 0, 128));
		CSS_COLORS.put("olive", new Color(null, 128, 128, 0));
		CSS_COLORS.put("purple", new Color(null, 128, 0, 128));
		CSS_COLORS.put("silver", ColorConstants.lightGray);
		CSS_COLORS.put("teal", new Color(null, 0, 128, 128));
	}

	static {
		SYSTEM_DEFAULT_COLORS.putAll(CSS_COLORS);

		SYSTEM_DEFAULT_COLORS.put("activeborder", Display.getCurrent()
				.getSystemColor(SWT.COLOR_WIDGET_BORDER));
		// Active window border.
		SYSTEM_DEFAULT_COLORS.put("activecaption",
				ColorConstants.titleBackground);
		// Active window caption.
		SYSTEM_DEFAULT_COLORS
				.put("appworkspace", ColorConstants.listBackground);
		// Background color of multiple document interface.
		SYSTEM_DEFAULT_COLORS.put("background", ColorConstants.listBackground);
		// Desktop background.
		SYSTEM_DEFAULT_COLORS.put("buttonface", ColorConstants.button);
		// Face color for three-dimensional display elements.
		SYSTEM_DEFAULT_COLORS.put("buttonhighlight",
				ColorConstants.buttonLightest);
		// Dark shadow for three-dimensional display elements (for edges facing
		// away from the light source).
		SYSTEM_DEFAULT_COLORS.put("buttonshadow", ColorConstants.buttonDarker);
		// Shadow color for three-dimensional display elements.
		SYSTEM_DEFAULT_COLORS.put("buttontext", Display.getCurrent()
				.getSystemColor(SWT.COLOR_WIDGET_FOREGROUND));
		// Text on push buttons.
		SYSTEM_DEFAULT_COLORS
				.put("captiontext", ColorConstants.titleForeground);
		// Text in caption, size box, and scrollbar arrow box.
		SYSTEM_DEFAULT_COLORS.put("graytext",
				ColorConstants.titleInactiveForeground);
		// Grayed (disabled) text. This color is set to #000 if the current
		// display driver does not support a solid gray color.
		SYSTEM_DEFAULT_COLORS.put("highlight",
				ColorConstants.menuBackgroundSelected);
		// Item(s) selected in a control.
		SYSTEM_DEFAULT_COLORS.put("highlighttext",
				ColorConstants.menuForegroundSelected);
		// Text of item(s) selected in a control.
		SYSTEM_DEFAULT_COLORS.put("inactiveborder", Display.getCurrent()
				.getSystemColor(SWT.COLOR_TITLE_INACTIVE_BACKGROUND_GRADIENT));
		// Inactive window border.
		SYSTEM_DEFAULT_COLORS.put("inactivecaption",
				ColorConstants.titleInactiveBackground);
		// Inactive window caption.
		SYSTEM_DEFAULT_COLORS.put("inactivecaptionText",
				ColorConstants.titleInactiveForeground);
		// Color of text in an inactive caption.
		SYSTEM_DEFAULT_COLORS.put("infobackground",
				ColorConstants.tooltipBackground);
		// Background color for tooltip controls.
		SYSTEM_DEFAULT_COLORS.put("infotext", ColorConstants.tooltipForeground);
		// Text color for tooltip controls.
		SYSTEM_DEFAULT_COLORS.put("menu", ColorConstants.menuBackground);
		// Menu background.
		SYSTEM_DEFAULT_COLORS.put("menutext", ColorConstants.menuForeground);
		// Text in menus.
		SYSTEM_DEFAULT_COLORS.put("scrollbar", Display.getCurrent()
				.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
		// Scroll bar gray area.
		SYSTEM_DEFAULT_COLORS.put("threeddarkshadow", Display.getCurrent()
				.getSystemColor(SWT.COLOR_WIDGET_DARK_SHADOW));
		// Dark shadow for three-dimensional display elements.
		SYSTEM_DEFAULT_COLORS.put("threedface", Display.getCurrent()
				.getSystemColor(SWT.COLOR_WIDGET_BACKGROUND));
		// Face color for three-dimensional display elements.
		SYSTEM_DEFAULT_COLORS.put("threedhighlight", Display.getCurrent()
				.getSystemColor(SWT.COLOR_WIDGET_HIGHLIGHT_SHADOW));
		// Highlight color for three-dimensional display elements.
		SYSTEM_DEFAULT_COLORS.put("threedlightshadow", Display.getCurrent()
				.getSystemColor(SWT.COLOR_WIDGET_LIGHT_SHADOW));
		// Light color for three-dimensional display elements (for edges facing
		// the light source).
		SYSTEM_DEFAULT_COLORS.put("threedshadow", Display.getCurrent()
				.getSystemColor(SWT.COLOR_WIDGET_DARK_SHADOW));
		// Dark shadow for three-dimensional display elements.
		SYSTEM_DEFAULT_COLORS.put("window", Display.getCurrent()
				.getSystemColor(SWT.COLOR_LIST_BACKGROUND));
		// Window background.
		SYSTEM_DEFAULT_COLORS.put("windowframe", Display.getCurrent()
				.getSystemColor(SWT.COLOR_WIDGET_BORDER));
		// Window frame.
		SYSTEM_DEFAULT_COLORS.put("windowtext", Display.getCurrent()
				.getSystemColor(SWT.COLOR_LIST_FOREGROUND));
		// Window text
		SYSTEM_DEFAULT_COLORS.put("hyperlink", ColorConstants.blue);
	}

	public static Color getCSSColor(String cssText) {
		return (Color) CSS_COLORS.get(cssText.toLowerCase());
	}

	public static Map getCSSColorMap() {
		return Collections.unmodifiableMap(CSS_COLORS);
	}

	public static RGB getColorRGB(String textColor) {
		if (textColor == null || textColor.length() == 0) {
			return null;
		}

		Color color = getCSSColor(textColor);
		if (color != null) {
			return color.getRGB();
		}

		if (textColor.charAt(0) == '#' && textColor.length() == 4) {
			char[] rgbChars = textColor.toCharArray();
			char[] fullChars = { rgbChars[0], rgbChars[1], rgbChars[1],
					rgbChars[2], rgbChars[2], rgbChars[3], rgbChars[3] };

			textColor = String.valueOf(fullChars);
		}

		if (textColor.charAt(0) == '#' && textColor.length() == 7) {
			try {
				int intColor = Integer.decode(textColor).intValue();

				if (intColor > 0xFFFFFF || intColor < 0) {
					return null;
				}
				int r = intColor >> 16;
				int g = (intColor >> 8) & 0xFF;
				int b = intColor & 0xFF;
				return new RGB(r, g, b);

			} catch (NumberFormatException e) {
				return null;
			}
		}
		return null;
	}

	public static String getStringColor(RGB rgb) {
		if (rgb == null) {
			return "";
		}

		StringBuffer buffer = new StringBuffer("#");
		int[] intRGBs = new int[] { rgb.red, rgb.green, rgb.blue };
		for (int i = 0; i < 3; i++) {
			if (intRGBs[i] < 16) {
				buffer.append("0");
			}
			buffer.append(Integer.toHexString(intRGBs[i]).toUpperCase());
		}
		return buffer.toString();
	}
}
