/*******************************************************************************
 * Copyright (c) 2006 Oracle Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Cameron Bateman/Oracle - initial API and implementation
 *    
 ********************************************************************************/

package org.eclipse.jst.jsf.validation.internal.el;

import java.util.Iterator;

import org.eclipse.core.resources.IFile;
import org.eclipse.jst.jsf.context.structureddocument.internal.provisional.IStructuredDocumentContext;
import org.eclipse.jst.jsf.context.structureddocument.internal.provisional.IStructuredDocumentContextFactory;
import org.eclipse.jst.jsf.validation.internal.el.diagnostics.ValidationMessageFactory;
import org.eclipse.jst.jsp.core.internal.regions.DOMJSPRegionContexts;
import org.eclipse.jst.jsp.core.internal.validation.JSPValidator;
import org.eclipse.wst.sse.core.internal.provisional.text.IStructuredDocument;
import org.eclipse.wst.sse.core.internal.provisional.text.ITextRegion;
import org.eclipse.wst.sse.core.internal.provisional.text.ITextRegionCollection;
import org.eclipse.wst.sse.core.internal.provisional.text.ITextRegionList;
import org.eclipse.wst.validation.internal.core.Message;
import org.eclipse.wst.validation.internal.provisional.core.IReporter;
import org.eclipse.wst.xml.core.internal.regions.DOMRegionContext;

/**
 * Validator for JSF expression language partitions in JSP document
 * 
 * @author cbateman
 *
 */
public class JSPELValidatorCopy extends JSPValidator
{
	/**
	 * Validates a particular region container
	 * 
	 * @param structuredDoc
	 * @param container
	 * @param reporter
	 * @param file
	 */
	protected void validateRegionContainer(IStructuredDocument structuredDoc, ITextRegionCollection container, IReporter reporter, IFile file) {

		ITextRegionCollection containerRegion = container;
		Iterator regions = containerRegion.getRegions().iterator();
		ITextRegion region = null;
		while (regions.hasNext() && !reporter.isCancelled()) {
			region = (ITextRegion) regions.next();
			
			if (region instanceof ITextRegionCollection) {
				ITextRegionCollection parentRegion = ((ITextRegionCollection) region);
				if (parentRegion.getType() == DOMRegionContext.XML_TAG_ATTRIBUTE_VALUE)
				{
					// look for attribute pattern "#{}"
					// TODO: need to generalize this for RValue concatenation
					final ITextRegionList  regionList = parentRegion.getRegions();
					if (regionList.size() >= 4)
					{
						ITextRegion  openQuote = regionList.get(0);
						ITextRegion  openVBLQuote = regionList.get(1);

						if (	(openQuote.getType() == DOMJSPRegionContexts.XML_TAG_ATTRIBUTE_VALUE_DQUOTE
									|| openQuote.getType() == DOMJSPRegionContexts.XML_TAG_ATTRIBUTE_VALUE_SQUOTE)
									&& (openVBLQuote.getType() == DOMJSPRegionContexts.JSP_VBL_OPEN))
						{
							// we appear to be inside "#{", so next should be a VBL_CONTENT if there's anything
							// here to validate
							final ITextRegion content = regionList.get(2);
							if (content.getType() == DOMJSPRegionContexts.JSP_VBL_CONTENT)
							{
                                final int contentStart = 
                                    parentRegion.getStartOffset(content);
                                final IStructuredDocumentContext context =
                                    IStructuredDocumentContextFactory.INSTANCE.
                                        getContext(structuredDoc, contentStart);
                                final ELExpressionValidator validator =
                                    new ELExpressionValidator
                                        (context, 
                                         parentRegion.getText(content),
                                         file);
                                validator.validateXMLNode();
								validator.reportFindings(this, reporter);
							}
							
							boolean foundClosingQuote = false;
							for (int i = 2; i < regionList.size(); i++)
							{
								ITextRegion  searchRegion = regionList.get(i);
								if (searchRegion.getType() == DOMJSPRegionContexts.JSP_VBL_CLOSE)
								{
									foundClosingQuote = true;
								}
							}
							
							if (!foundClosingQuote)
							{
								final int problemStartOffset = parentRegion.getStartOffset();
                                final int length = parentRegion.getLength();
								final Message message = 
                                    ValidationMessageFactory.create_MISSING_CLOSING_EXPR_BRACKET(problemStartOffset, length, file);
								reporter.addMessage(this, message);
							}
						}
					}
					
//					Iterator childRegions = parentRegion.getRegions().iterator();
//					while (childRegions.hasNext() && !reporter.isCancelled()) {
//						ITextRegion childRegion = (ITextRegion) childRegions.next();
//						if (childRegion.getType() == DOMJSPRegionContexts.JSP_EL_CONTENT)
//							validateXMLNode(parentRegion, childRegion, reporter, file);
//					}
				}
			}
		}
	}


	public void validateFile(IFile file, IReporter reporter) 
    {
        // do nothing
//		IStructuredModel model = null;
//		try {
//			model = StructuredModelManager.getModelManager().getModelForRead(file);
//			DOMModelForJSP jspModel = (DOMModelForJSP) model;
//			IStructuredDocument structuredDoc = jspModel.getStructuredDocument();
//			IStructuredDocumentRegion curNode = structuredDoc.getFirstStructuredDocumentRegion();
//
//			while (null != (curNode = curNode.getNext()) && !reporter.isCancelled()) {
//				if (curNode.getType() != DOMRegionContext.XML_COMMENT_TEXT && curNode.getType() != DOMRegionContext.XML_CDATA_TEXT && curNode.getType() != DOMRegionContext.UNDEFINED) {
//					validateRegionContainer(structuredDoc, curNode, reporter, file);
//				}
//			}
//		}
//		catch (Exception e) {
//			e.printStackTrace(System.err);
//		}
//		finally {
//			if (null != model)
//				model.releaseFromRead();
//		}
	}
}
